/*
BTCL - Copyright (C) 2001, 2002
by Andrea Marchesini
a.marchesini@ngi.it

MYSQL.C
*/

#include <stdlib.h>
#include "btcl.h"

/* Var for DB: */
MYSQL mysql[MAX_DB];
MYSQL_RES *mysql_result[MAX_DB];
int mysql_status_db[MAX_DB];

/*
db_num_rows passandogli un puntatore a DB mi ritorna il numero delle linee
della query effettutata.
*/
int mysql_db_num_rows(ClientData clientdata, Tcl_Interp *interp, int argc, 
                      char *argv[]) {
 int db;
 char num_rows[10];

 if(argc!=2) {
  puts("Error: wrong args: should be \"mysql_db_num_rows VarDb\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(mysql_status_db[db]<2) {
  if(mysql_status_db[db]==0) puts("Error: no connect to db");
  if(mysql_status_db[db]==1) puts("Error: no query to db");
  return TCL_ERROR;
 }

 sprintf(num_rows,"%d",(int)mysql_num_rows(mysql_result[db]));
 Tcl_SetResult (interp, num_rows, TCL_VOLATILE);
 return TCL_OK;
}

/*
 * Mi da il codice dell'errore come output oppure mi da la descrizione in
 * una var passata
 */
int mysql_db_error(ClientData clientdata, Tcl_Interp *interp, int argc,
                   char *argv[]) {
 int i;
 char *tmp;

 if(argc>3 || argc<2) {
  puts("Error: wrong args: should be \"mysql_db_error VarDb ?VarError?\"");
  return TCL_ERROR;
 }
 i=atoi(argv[1]);
 if(i<0 || i>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }
 if(mysql_status_db[i]==0) {
  puts("Error: No connect to DB");
  return TCL_ERROR;
 }
 if(argc==3) Tcl_SetVar(interp,argv[2],mysql_error(&mysql[i]),0);
 
 tmp=(char *)malloc(mysql_errno(&mysql[i])*sizeof(int));

 sprintf(tmp,"%d",mysql_errno(&mysql[i]));
 Tcl_SetResult (interp, tmp, TCL_VOLATILE);
 return TCL_OK;
}

/*
 * Chiude un DB. Riceve un solo argomento che  la variabile contenente il DB.
 */
int mysql_db_close(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int i;
 if(argc!=2) {
  puts("Error: wrong args: should be \"mysql_db_close VarDb\"");
  return TCL_ERROR;
 }
 if(Tcl_GetVar(interp,argv[1],0)!=NULL) i=atoi(Tcl_GetVar(interp,argv[1],0));
 else {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(Tcl_UnsetVar(interp,argv[1],0)<0) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }
 if(i<0 || i>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }
 mysql_status_db[i]=0;
 mysql_close(&mysql[i]);
 return TCL_OK;
}

/*
 * Legge riga dal DB. Come argomento vuole una var che poi trasformer in
 * array con tutti gli elementi per riga.
 * Restituisce come valore il numero di elementi per riga che  riuscito a
 * prendere altimenti -1 =)
 */
int mysql_db_read(ClientData clientdata, Tcl_Interp *interp, int argc, 
                  char *argv[]) {
 MYSQL_ROW row;
 int db;
 unsigned int num_fields,i;
 char *tmp;

 if(argc!=3) {
  puts("Error: wrong args: should be \"mysql_db_read VarDb NameArray\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(mysql_status_db[db]<2) {
  if(mysql_status_db[db]==0) puts("Error: no connect to db");
  if(mysql_status_db[db]==1) puts("Error: no query to db");
  return TCL_ERROR;
 }

 num_fields=mysql_num_fields(mysql_result[db]);
 if((row=mysql_fetch_row(mysql_result[db]))) {
  for(i=0;i<num_fields;i++) {
   
   tmp=(char *)malloc(sizeof(argv[2])+10);
   
   sprintf(tmp,"%s(%d)",argv[2],i);
   Tcl_SetVar(interp,tmp,row[i],0);
  }
  sprintf(tmp,"%d",num_fields);
 } else {
  Tcl_SetResult (interp, "-1", TCL_VOLATILE);
  return TCL_OK;
 }
 Tcl_SetResult (interp, tmp, TCL_VOLATILE);
 return TCL_OK;
}

/* Quera il DB. Riceve come variabile solo un argomento. */
int mysql_db_query(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int db;

 if(argc!=3) {
  puts("Error: wrong args: should be \"mysql_db_query VarDb String\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(mysql_status_db[db]==0) {
  puts("Error: no connect to db");
  return TCL_ERROR;
 }

 if(mysql_query(&mysql[db],argv[2])) {
  Tcl_SetResult (interp, "-1", TCL_VOLATILE);
  return TCL_OK;
 }
 mysql_result[db]=mysql_store_result(&mysql[db]);
 mysql_status_db[db]=2;
 Tcl_SetResult (interp, "0", TCL_VOLATILE);
 return TCL_OK;
}

/*
 * Connect_db si connette ad un DB in mysql
 * Riceve come argomenti quelli della struttura qui sotto =)
 */
int mysql_db_connect(ClientData clientdata, Tcl_Interp *interp, int argc, 
                     char *argv[]) {
 char tmp[10];
 int i;
/*
  Ma che bella struttura:

  MYSQL *mysql, const char *host,
  const char *user,
  const char *passwd,
  const char *db,
  unsigned int port,
  const char *unix_socket,
  unsigned int clientflag
*/
 if(argc<5 || argc>8) {
  puts("Error: wrong args: should be \"mysql_db_connect host user password \
database ?port? ?unix_socket? ?clientflag?\"");
  return TCL_ERROR;
 }
 for(i=0;i<MAX_DB;i++)
  if(mysql_status_db[i]==0) break;
  if(i==MAX_DB) {
   puts("Error: wrong args: max database open");
   return TCL_ERROR;
  }

 mysql_init(&mysql[i]);
 switch(argc) {
  case 5:
   if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],0,NULL,
                          0)) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 6:
   if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],
                          atoi(argv[5]),NULL,0)) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 7:
   if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],
                          atoi(argv[5]),argv[6],0))  {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  default:
   if(argv[7]=="FOUND_ROWS") {
    if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],
                           atoi(argv[5]),argv[6],CLIENT_FOUND_ROWS)) {
     Tcl_SetResult (interp, "-1", TCL_VOLATILE);
     return TCL_OK;
    }
   } else if(argv[7]=="NO_SCHEMA") {
    if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],
                           atoi(argv[5]),argv[6],CLIENT_NO_SCHEMA)) {
     Tcl_SetResult (interp, "-1", TCL_VOLATILE);
     return TCL_OK;
    }
   } else if(argv[7]=="ODBC") {
    if(!mysql_real_connect(&mysql[i],argv[1],argv[2],argv[3],argv[4],
                           atoi(argv[5]),argv[6],CLIENT_ODBC)) {
     Tcl_SetResult (interp, "-1", TCL_VOLATILE);
     return TCL_OK;
    }
   } else {
    puts("Error: wrong args: clientflag is NO_SCHEMA, ODBC or FOUND_ROWS");
    return TCL_ERROR;
   }
   break;
 }
 mysql_status_db[i]=1;
 sprintf(tmp,"%d",i);
 Tcl_SetResult (interp, tmp, TCL_VOLATILE);
 return TCL_OK;
}

/* 
 * Mi riempe un array passato come argomento con tutte le info del caso
 * sulla connessione al Db.
 */
int mysql_db_info(ClientData clientdata, Tcl_Interp *interp, int argc,
                  char *argv[]) {
 char *tmp;
 int db;

 if(argc!=3) {
  puts("Error: wrong args: should be \"mysql_db_info VarDb NameVar\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(mysql_status_db[db]==0) {
  puts("Error: no connect to db");
  return TCL_ERROR;
 }
 
 /*
 SERVER - CLIENT - HOST - PROTO
 */

 tmp=(char *)malloc(sizeof(argv[2])+10);

 sprintf(tmp,"%s(SERVER)",argv[2]);
 Tcl_SetVar(interp,tmp,mysql_get_server_info(&mysql[db]),0);
 sprintf(tmp,"%s(CLIENT)",argv[2]);
 Tcl_SetVar(interp,tmp,mysql_get_client_info(),0);
 sprintf(tmp,"%s(HOST)",argv[2]);
 Tcl_SetVar(interp,tmp,mysql_get_host_info(&mysql[db]),0);
 sprintf(tmp,"%s(PROTO)",argv[2]);
 
 Tcl_SetResult (interp, "4", TCL_VOLATILE);
 return TCL_OK;
}
