/*
BTCL - Copyright (C) 2001, 2002
by Andrea Marchesini
a.marchesini@ngi.it

PGSQL.H
*/

#include <stdlib.h>
#include <string.h>
#include "btcl.h"

/* Var for DB: */
PGconn *pgsql[MAX_DB];
PGresult *pgsql_result[MAX_DB];
int pgsql_status_db[MAX_DB];

/*
pgsql_db_num_rows passandogli un puntatore a DB mi ritorna il numero delle 
linee della query effettutata.
*/
int pgsql_db_num_rows(ClientData clientdata, Tcl_Interp *interp, int argc, 
                      char *argv[]) {
 int db;
 char num_rows[10];

 if(argc!=2) {
  puts("Error: wrong args: should be \"pgsql_db_num_rows VarDb\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(pgsql_status_db[db]<2) {
  if(pgsql_status_db[db]==0) puts("Error: no connect to db");
  if(pgsql_status_db[db]==1) puts("Error: no query to db");
  return TCL_ERROR;
 }

 sprintf(num_rows,"%d",PQntuples(pgsql_result[db]));
 Tcl_SetResult (interp, num_rows, TCL_VOLATILE);
 return TCL_OK;
}

/* Mi da la descrizione dell'errore in una var puntatore a DB. */
int pgsql_db_error(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int i;

 if(argc>3 || argc<2) {
  puts("Error: wrong args: should be \"pgsql_db_error VarDb\"");
  return TCL_ERROR;
 }
 i=atoi(argv[1]);
 if(i<0 || i>MAX_DB) {
  puts("Error: the VarDb");
  return TCL_ERROR;
 }
 if(pgsql_status_db[i]==0) {
  puts("Error: No connect to DB");
  return TCL_ERROR;
 }
 Tcl_SetResult (interp,PQerrorMessage(pgsql[i]), TCL_VOLATILE);
 return TCL_OK;
}

/*
 * Chiude un DB. Riceve un solo argomento che e' la variabile contenente il DB.
 */
int pgsql_db_close(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int i;

 if(argc!=2) {
  puts("Error: wrong args: should be \"pgsql_db_close VarDb\"");
  return TCL_ERROR;
 }
 if(Tcl_GetVar(interp,argv[1],0)!=NULL) i=atoi(Tcl_GetVar(interp,argv[1],0));
 else {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(Tcl_UnsetVar(interp,argv[1],0)<0) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }
 if(i<0 || i>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }
 pgsql_status_db[i]=0;
 PQfinish(pgsql[i]);
 return TCL_OK;
}

/*
 * Legge riga dal DB. Come argomento vuole una var che poi trasformer in
 * array con tutti gli elementi per riga e il numero di riga che si vuole
 * leggere.
 * Restituisce come valore il numero di elementi per riga che  riuscito a
 * prendere altimenti -1 =)
 */
int pgsql_db_read(ClientData clientdata, Tcl_Interp *interp, int argc, 
                  char *argv[]) {
 int db;
 unsigned int num_fields,i;
 char *tmp;

 if(argc!=4) {
  puts("Error: wrong args: should be \"pgsql_db_read VarDb NameArray \
NumberLine\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(pgsql_status_db[db]<2) {
  if(pgsql_status_db[db]==0) puts("Error: no connect to db");
  if(pgsql_status_db[db]==1) puts("Error: no query to db");
  return TCL_ERROR;
 }

 num_fields = PQnfields(pgsql_result[db]);

 for (i = 0; i < num_fields; i++) {
  tmp=(char *)malloc(sizeof(argv[2])+10);
  sprintf(tmp,"%s(%d)",argv[2],i);
  Tcl_SetVar(interp,tmp,PQgetvalue(pgsql_result[db], atoi(argv[3]), i),0);
 }
 
 tmp=(char *)malloc(10);
 
 sprintf(tmp,"%d",num_fields);
 Tcl_SetResult (interp, tmp, TCL_VOLATILE);
 return TCL_OK;
}

/* Quera il DB. Riceve come variabile solo un argomento. */
int pgsql_db_query(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int db;

 if(argc!=3) {
  puts("Error: wrong args: should be \"pgsql_db_query VarDb String\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(pgsql_status_db[db]==0) {
  puts("Error: no connect to db");
  return TCL_ERROR;
 }

 pgsql_result[db]=PQexec(pgsql[db],argv[2]);
 if(strlen(PQerrorMessage(pgsql[db]))>0) Tcl_SetResult (interp,"-1", 
           TCL_VOLATILE);
 else {
  Tcl_SetResult (interp,"0", TCL_VOLATILE);
  pgsql_status_db[db]=2;
 }
 return TCL_OK;
}

/* 
 * Connect_db si connette ad un DB in pgsql
 * Riceve come argomenti quelli della struttura qui sotto =)
 */
int pgsql_db_connect(ClientData clientdata, Tcl_Interp *interp, int argc, 
                     char *argv[]) {
 char tmp[10];
 int i;

 /*
 const char *pghost, 
 const char *pgport,
 const char *pgoptions, 
 const char *pgtty,
 const char *dbName,
 const char *login, 
 const char *pwd
 */
	  
 if(argc<3 || argc>8) {
  puts("Error: wrong args: should be \"pgsql_db_connect host ?user? ?passwd? \
database port ?opt? ?pgtty?\"");
  return TCL_ERROR;
 }
 for(i=0;i<MAX_DB;i++)
  if(pgsql_status_db[i]==0) break;
  if(i==MAX_DB) {
   puts("Error: wrong args: max database open");
   return TCL_ERROR;
  }

 switch(argc) {
  case 4:
   pgsql[i]=PQsetdbLogin(argv[1],argv[3],NULL,NULL,argv[2],NULL,NULL);
   if(PQstatus(pgsql[i])==CONNECTION_BAD) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 5:
   pgsql[i]=PQsetdbLogin(argv[1],argv[4],NULL,NULL,argv[3],argv[2],NULL);
   if(PQstatus(pgsql[i])==CONNECTION_BAD) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 6:
   pgsql[i]=PQsetdbLogin(argv[1],argv[5],NULL,NULL,argv[4],argv[2],argv[3]);
   if(PQstatus(pgsql[i])==CONNECTION_BAD) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 7:
   pgsql[i]=PQsetdbLogin(argv[1],argv[5],argv[6],NULL,argv[4],argv[2],argv[3]);
   if(PQstatus(pgsql[i])==CONNECTION_BAD) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
  case 8:
   pgsql[i]=PQsetdbLogin(argv[1],argv[5],argv[6],argv[7],argv[4],argv[2],
                         argv[3]);
   if(PQstatus(pgsql[i])==CONNECTION_BAD) {
    Tcl_SetResult (interp, "-1", TCL_VOLATILE);
    return TCL_OK;
   }
   break;
 }
 pgsql_status_db[i]=1;
 sprintf(tmp,"%d",i);
 Tcl_SetResult (interp, tmp, TCL_VOLATILE);
 return TCL_OK;
}

/* Flushia una variabile risultato: */
int pgsql_db_clean(ClientData clientdata, Tcl_Interp *interp, int argc, 
                   char *argv[]) {
 int db;

 if(argc!=2) {
  puts("Error: wrong args: should be \"pgsql_db_clear VarDb\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(pgsql_status_db[db]==0) {
  puts("Error: no connect to db");
  return TCL_ERROR;
 }
 pgsql_status_db[db]=1;
 PQclear(pgsql_result[db]);
 return TCL_OK;
}

/*
 * Mi riempe un array passato come argomento con tutte le info del caso
 * sulla connessione al Db.
 */
int pgsql_db_info(ClientData clientdata, Tcl_Interp *interp, int argc, 
                  char *argv[]) {
 int db;
 char *tmp;

 if(argc!=3) {
  puts("Error: wrong args: should be \"pgsql_db_info VarDb NameVar\"");
  return TCL_ERROR;
 }
 db=atoi(argv[1]);
 if(db<0 || db>MAX_DB) {
  puts("Error: VarDb");
  return TCL_ERROR;
 }

 if(pgsql_status_db[db]==0) {
  puts("Error: no connect to db");
  return TCL_ERROR;
 }
 
 /*
 DATABASE - USER - PASSWORD - HOST - PORT - TTY - OPTIONS
 */

 tmp=(char *)malloc(sizeof(argv[2])+10);

 sprintf(tmp,"%s(DATABASE)",argv[2]);
 Tcl_SetVar(interp,tmp,PQdb(pgsql[db]),0);
 sprintf(tmp,"%s(USER)",argv[2]);
 Tcl_SetVar(interp,tmp,PQuser(pgsql[db]),0);
 sprintf(tmp,"%s(PASSWORD)",argv[2]);
 Tcl_SetVar(interp,tmp,PQpass(pgsql[db]),0);
 sprintf(tmp,"%s(HOST)",argv[2]);
 Tcl_SetVar(interp,tmp,PQhost(pgsql[db]),0);
 sprintf(tmp,"%s(PORT)",argv[2]);
 Tcl_SetVar(interp,tmp,PQport(pgsql[db]),0);
 sprintf(tmp,"%s(TTY)",argv[2]);
 Tcl_SetVar(interp,tmp,PQtty(pgsql[db]),0);
 sprintf(tmp,"%s(OPTIONS)",argv[2]);
 Tcl_SetVar(interp,tmp,PQoptions(pgsql[db]),0);
 
 Tcl_SetResult (interp, "8", TCL_VOLATILE);
 return TCL_OK;
}
